#ifndef BURN_INTF_H__
#define BURN_INTF_H__
#include "l502api.h"
#include "firm_check.h"

typedef struct st_burn_intf t_burn_intf;


typedef enum
{
    L502BURN_ERR_INVALID_USAGE          = -10000,
    L502BURN_ERR_FILE_OPEN              = -10001,
    L502BURN_ERR_FILE_READ              = -10002,
    L502BURN_ERR_INVALID_BOOT_FLAG      = -10003,
    L502BURN_ERR_READ_FILE              = -10004,
    L502BURN_ERR_BYTE_VEREFY            = -10005,
    L502BURN_ERR_WRITE_FILE             = -10006,
    L502BURN_ERR_CANNOT_OPEN_DEVICE     = -10007,
    L502BURN_ERR_INVALID_INTERFACE      = -10008,
    L502BURN_ERR_MEMORY_ALLOC           = -10009,
    L502BURN_ERR_INVALID_SIGN           = -10010,
    L502BURN_ERR_FIRM_PARSE             = -10011,
    L502BURN_ERR_FIRM_PARSE_ROOT_ELEM   = -10012,
    L502BURN_ERR_FIRM_PARSE_NO_DATA     = -10013,
    L502BURN_ERR_FIRM_DEV_CHECK         = -10014,
    L502BURN_ERR_FIRM_OLD_VERSION       = -10015
} t_l502_burn_errs;



typedef enum
{
    L502BURN_OP_WRITE,
    L502BURN_OP_READ
} t_l502burn_op;

typedef struct
{
    const char* serial;
    int port;
    t_l502burn_op op;
    const char* filename;
    t_lfirm_info_ptr firm_info;
    int read_start;
    int read_last;
    int invers;
    int all;
    int ver_dont_check;
    int main_only;
    int reserv_only;
    const t_burn_intf* intf;

} t_l502_burn_opt;

typedef int (*t_exec_cmd_cb)(t_l502_burn_opt* opts);

typedef int (*t_intf_open)(t_l502_burn_opt* opts);
typedef int (*t_intf_close)(void);
typedef int (*t_check_dev)(t_l502_burn_opt *opts);
typedef int (*t_check_version)(t_l502_burn_opt *opts, const char* version);
typedef int (*t_intf_flash_set_prot)(uint8_t prot);
typedef int (*t_intf_flash_erase_64k)(uint32_t addr);
typedef int (*t_intf_flash_write_start)(uint32_t addr, const uint8_t* data, uint32_t size);
typedef int (*t_intf_flash_write_next)( const uint8_t* data, uint32_t size);
typedef int (*t_intf_flash_write_stop)(void);
typedef int (*t_intf_flash_read)(uint32_t addr, uint8_t *data, uint32_t size);
typedef int (*t_intf_exec_for_all)(t_l502_burn_opt* opts, t_exec_cmd_cb cb);




struct st_burn_intf
{
    const char* name;
    t_intf_open open;
    t_intf_close close;
    t_check_dev check_dev;
    t_check_version check_ver;
    t_intf_flash_set_prot flash_set_prot;
    t_intf_flash_erase_64k flash_erase_64k;
    t_intf_flash_write_start flash_write_start;
    t_intf_flash_write_next flash_write_next;
    t_intf_flash_write_stop flash_write_stop;
    t_intf_flash_read flash_read;
    t_intf_exec_for_all exec_for_all;
};



typedef int (*t_write_firm)(t_l502_burn_opt* opts);

typedef struct st_burn_dev
{
    const char* name;
    t_write_firm* write_firm;
} t_burn_dev;



#endif
