#include <getopt.h>
#include "l502api.h"
#include <stdio.h>
#include <string.h>

#include "burn_intf.h"
#include "firm_check.h"
//#include "config.h"

#define L502BURN_OPT_HELP           'h'
#define L502BURN_OPT_INTF           'i'
#define L502BURN_OPT_READ           'r'
#define L502BURN_OPT_IOADDR         'a'
#define L502BURN_OPT_READ_START     'f'
#define L502BURN_OPT_READ_LAST      'l'
#define L502BURN_OPT_SERIAL         's'
#define L502BURN_OPT_INVERSE        0x10000
#define L502BURN_OPT_MAIN_ONLY      0x10001
#define L502BURN_OPT_ALL            0x10002
#define L502BURN_OPT_DONT_CHECK_VER 0x10003
#define L502BURN_OPT_RESERV_ONLY    0x10004

static const struct option f_long_opt[] =
{
    {"help",        no_argument,        0, L502BURN_OPT_HELP},
    {"intf",        required_argument,  0, L502BURN_OPT_INTF},
    {"read",        no_argument,        0, L502BURN_OPT_READ},
    {"first",       required_argument,  0, L502BURN_OPT_READ_START},
    {"last",        required_argument,  0, L502BURN_OPT_READ_LAST},
    {"io-addr",     required_argument,  0, L502BURN_OPT_IOADDR},
    {"serial",      required_argument,  0, L502BURN_OPT_SERIAL},
    {"bit-inverse", no_argument,        0, L502BURN_OPT_INVERSE},
    {"main-only",   no_argument,        0, L502BURN_OPT_MAIN_ONLY},
    {"all",         no_argument,        0, L502BURN_OPT_ALL},
    {"dont-check-ver", no_argument,     0, L502BURN_OPT_DONT_CHECK_VER},
    {"reserv-only", no_argument,        0, L502BURN_OPT_RESERV_ONLY},
    {0,0,0,0}
};



static const char* f_opt_str = "hri:a:f:l:s:";

#if 0
static const char* f_usage_descr = \
"\nUsage:\n"\
"l502burn --intf=lpt --io-addr=io --main-only file\n"\
"Burn firmware image from file to main area of SST25 flash and verefy\n\n" \
"l502burn --intf=lpt --io-addr=io -r file\n"\
"Read entire SST25 flash content and write it to file\n\n" \
"Options:\n" \
"-a, --io-addr=io       - specify LPT first io address\n"\
"    --bit-inverse      - inverse bits for read (for comparsion with firmware)"\
"-f, --first=addr       - first read address (if -r specified)\n"\
"-h, --help             - display this help and exit\n"\
"-i, --intf=interface   - specify interface (lpt, lpcie)\n"\
"-l, --last=addr        - last read address (if -r specified)\n"\
"    --main-only        - write only main copy of firmware (reserved copy leave untouched)"\
"-r, --read             - read form flash rather than burn\n"\
"-s, --serial           - device serial for pcie interface";
#endif

static const char* f_usage_descr = \
"\nUsage:\n"\
"lpcie_firm_update --serial=<sn> file\n\n"\
"Burn firmware from file to device with specified serial\n\n"\
"lpcie_firm_update --all file\n\n"\
"Burn firmware from file to all suitued devices in system\n\n";

extern t_burn_intf g_pcie_intf;

static const t_burn_intf *f_intfs[] =
{
    &g_pcie_intf
};




typedef struct
{
    uint32_t size;
    uint32_t flags;
    uint32_t res[2];
} t_firm_prefix;

/** биты регистра статуса */
typedef enum
{
    SST25_STATUS_BUSY   = 0x01,
    SST25_STATUS_WEL    = 0x02,
    SST25_STATUS_BP0    = 0x04,
    SST25_STATUS_BP1    = 0x08,
    SST25_STATUS_BP2    = 0x10,
    SST25_STATUS_BP3    = 0x20,
    SST25_STATUS_AAI    = 0x40,
    SST25_STATUS_BPL    = 0x80
} t_sst25_status_bits;


#define L502_FLASH_FIRM_PREFIX  0x100000
#define L502_FLASH_FIRM_START   0x100010
#define L502_FLASH_FIRM_LAST    0x16FFFF
#define L502_FLASH_BOOT_FLAG    0x170000
#define L502_FLASH_SIZE         0x200000



#define L502BURN_BLOCK_SIZE 4096
static uint8_t f_block[L502BURN_BLOCK_SIZE], f_block2[L502BURN_BLOCK_SIZE];


typedef struct
{
    uint32_t prefix;
    uint32_t firm;
    uint32_t last;
} t_firm_addrs;

static t_firm_addrs f_firm_addrs[2] =
{
    {0x180000, 0x180010, 0x1EFFFF},
    {0x100000, 0x100010, 0x16FFFF}
};




static void f_print_usage(void)
{
    printf("%s", f_usage_descr);
}


static const char* f_err_str[] =
{
    "Invalid usage. Try lpcie_firm_update --help for more information",
    "Cannot open specified file",
    "Invalid bootflag value",
    "Read file error",
    "Verefy byte error",
    "Write file error",
};

static void f_print_err(int err, const char* msg)
{
    const char* err_str = "Unknown error code";
    /*if (msg)
        fprintf(stderr, "%s\n", msg);

    if (err > -100)
        err_str = sst25_get_err_string(err);
    else
    {
        int suberr = -(err + 100);
        if (suberr < sizeof(f_err_str)/sizeof(f_err_str[0]))
            err_str = f_err_str[suberr];
        else
            err_str = "Unknown error code";
    }*/
    printf("%s. Error %d!\n",  msg, err);
}



static int f_parse_options(t_l502_burn_opt* st, int argc, char** argv, int *out)
{
    int err = 0;
    int opt = 0;

    *out = 0;
    st->port = 0x378;
    st->op = L502BURN_OP_WRITE;
    st->read_start = 0;
    st->read_last = L502_FLASH_SIZE-1;
    st->invers  = 0;
    st->serial = 0;
    st->intf = f_intfs[0];
    st->main_only = 0;
    st->all = 0;
    st->ver_dont_check = 0;
    st->reserv_only = 0;




    /************************** разбор опций *****************************/
    while ((opt!=-1) && !err && !*out)
    {
        int ind;
        opt = getopt_long(argc, argv, f_opt_str,
                          f_long_opt, &ind);

        switch (opt)
        {
            case -1:
                break;
            case L502BURN_OPT_HELP:
                f_print_usage();
                *out = 1;
                break;
            case L502BURN_OPT_INVERSE:
                st->invers = 1;
                break;
            case L502BURN_OPT_IOADDR:
                sscanf(optarg, "%i", &st->port);
                if ((st->port < 0) || (st->port>=64*1024))
                {
                    err = L502BURN_ERR_INVALID_USAGE;
                    f_print_err(err, "Invalid lpt io address");
                }
                break;
            case L502BURN_OPT_READ:
                st->op = L502BURN_OP_READ;
                break;
            case L502BURN_OPT_READ_START:
                sscanf(optarg, "%i", &st->read_start);
                if ((st->read_start < 0) || (st->read_start>=L502_FLASH_SIZE))
                {
                    err = L502BURN_ERR_INVALID_USAGE;
                    f_print_err(err, "Invalid read start address");
                }
                break;
            case L502BURN_OPT_READ_LAST:
                sscanf(optarg, "%i", &st->read_last);
                if ((st->read_last < 0) || (st->read_last>=L502_FLASH_SIZE))
                {
                    err = L502BURN_ERR_INVALID_USAGE;
                    f_print_err(err, "Invalid read last address");
                }
                break;
            case L502BURN_OPT_INTF:
                {
                    unsigned int fnd = 0, i;
                    for (i=0; !fnd && (i<sizeof(f_intfs)/sizeof(f_intfs[0])); i++)
                    {
                        if (!strcmp(f_intfs[i]->name, optarg))
                        {
                            fnd = 1;
                            st->intf = f_intfs[i];
                        }
                    }
                    if (!fnd)
                    {
                        err = L502BURN_ERR_INVALID_INTERFACE;
                        f_print_err(err, "Invalid interface!");
                    }
                }
                break;
            case L502BURN_OPT_SERIAL:
                st->serial = optarg;
                break;
            case L502BURN_OPT_ALL:
                st->all = 1;
                break;
            case L502BURN_OPT_DONT_CHECK_VER:
                st->ver_dont_check = 1;
                break;
            case L502BURN_OPT_MAIN_ONLY:
                st->main_only = 1;
                break;
            case L502BURN_OPT_RESERV_ONLY:
                st->reserv_only = 1;
                break;
            default:
                err = L502BURN_ERR_INVALID_USAGE;
                f_print_err(err, 0);
                break;
        }
    }

    if (!*out && !err)
    {
        if (st->read_last < st->read_start)
        {
             err = L502BURN_ERR_INVALID_USAGE;
             f_print_err(err, "last read address must be >= start read address");
        }
        else
        {
            if ((argc-optind) < 1)
            {
                err = L502BURN_ERR_INVALID_USAGE;
                f_print_err(err, 0);
            }
            else
            {
                st->filename = argv[optind];
            }
        }
    }
    return err;
}


static unsigned char f_inverse_byte(unsigned char byte)
{
    unsigned char res = 0;
    int j;
    for (j=0; j < 8; j++)
    {
        if (byte & (1 << j))
            res |= (1 << (7-j));
    }
    return res;
}

static int f_erase(const t_l502_burn_opt* opts, uint32_t first, uint32_t last)
{
    int err = 0;
    uint32_t addr;
    for (addr=first; (addr < last) && !err; addr+=64*1024)
    {
        err = opts->intf->flash_erase_64k(addr);
    }
    if (err)
        f_print_err(err, "Erase flash error!");
    else
        printf("Flash erased successfully\n");
    return err;
}

static int f_write(const t_l502_burn_opt* opts, const t_firm_prefix* prefix,
                   uint32_t addr, uint32_t size)
{
    unsigned char last[2], wr_last = 0;
    /* запись... */

    int err = opts->intf->flash_write_start(addr, (const uint8_t*)prefix,
                                            sizeof(t_firm_prefix));
    size_t offs = 0;
    while (!err && size)
    {
        uint32_t wr_size = (uint32_t)firm_read(opts->firm_info, f_block, offs, L502BURN_BLOCK_SIZE);
        if (wr_size)
        {
            uint32_t i;
            offs+=wr_size;
            for (i=0; i < wr_size; i++)
            {
                f_block[i] = f_inverse_byte(f_block[i]);
            }

            if (wr_size & 1)
            {
                wr_last = 1;
                last[0] = f_block[wr_size-1];
                last[1] = 0xFF;
                wr_size-=1;
            }

            err = opts->intf->flash_write_next(f_block, wr_size);
            if (!err)
                printf(".");
            else
                printf("!");
            fflush(stdout);

        }
        size-=wr_size;

        if (!err && wr_last)
        {
            err = opts->intf->flash_write_next(last, 2);
            size--;
        }
    }


    opts->intf->flash_write_stop();
    printf("\n");
    if (err)
        f_print_err(err, "Burn firmware error!");
    else
        printf("Firmware burn done\n");

    return err;
}

static int f_firm_check(const t_l502_burn_opt* opts, const t_firm_prefix* prefix,
                   uint32_t addr, uint32_t size)
{
    int err = 0;
    uint32_t i;
    t_firm_prefix check_pref;


    /* проверяем префикс */
    err = opts->intf->flash_read(addr, (uint8_t*)&check_pref, sizeof(t_firm_prefix));

    for (i=0; (i < sizeof(check_pref))&&!err; i++)
    {
        if (((char*)&check_pref)[i]!=((char*)prefix)[i])
        {
            err = L502BURN_ERR_BYTE_VEREFY;
            printf("\nVerifying error! write 0x%02x, read 0x%02x, addr = 0x%06x\n",
                ((uint8_t*)prefix)[i], ((uint8_t*)&check_pref)[i], addr + i);
        }
    }

    if (!err)
    {
        size_t offs = 0;
        addr+=sizeof(t_firm_prefix);

        while (!err && size)
        {
            uint32_t wr_size = (uint32_t)firm_read(opts->firm_info, f_block, offs, L502BURN_BLOCK_SIZE);
            if (wr_size)
            {
                offs+=wr_size;

                err =  opts->intf->flash_read(addr, f_block2, wr_size);
                if (!err)
                {
                    for (i=0; (i < wr_size)&&!err; i++)
                    {
                        f_block2[i] = f_inverse_byte(f_block2[i]);
                        if (f_block[i]!=f_block2[i])
                        {
                            err = L502BURN_ERR_BYTE_VEREFY;
                            printf("Verifying error! write 0x%02x, read 0x%02x, addr = 0x%06x\n",
                                f_block[i], f_block2[i], addr + i);
                        }
                    }

                    if (!err)
                        printf(".");
                }
            }

            size-=wr_size;
            addr+=wr_size;
        }

        printf("\n");
    }
    if (err)
        f_print_err(err, 0);
    else
        printf("Firmware verifying done sucessfully\n");
    return err;
}


static int f_set_boot_flag(const t_l502_burn_opt* opts, uint8_t flag)
{
    int err = opts->intf->flash_erase_64k(L502_FLASH_BOOT_FLAG);
    if (!err)
    {
        uint8_t pref[2] = {flag,0};
        err = opts->intf->flash_write_start(L502_FLASH_BOOT_FLAG, pref, 2);
        if (!err)
            err = opts->intf->flash_write_stop();
    }

    if (!err)
    {
        uint8_t pref;
        err = opts->intf->flash_read(L502_FLASH_BOOT_FLAG, &pref,1);
        if (pref!=flag)
            err = L502BURN_ERR_INVALID_BOOT_FLAG;
    }

    if (err)
        f_print_err(err, "Set bootflag error!");
    else
        printf("Set bootflag sucessfully\n");
    return err;
}

static int f_get_boot_flag(const t_l502_burn_opt* opts, uint8_t* flag)
{
    int err = opts->intf->flash_read(L502_FLASH_BOOT_FLAG, flag, 1);
    if (!err)
        *flag&=1;
    return err;
}





static int f_write_firm_par(const t_l502_burn_opt* opts,
                           t_firm_prefix pref)
{
    int size = pref.size;
    t_firm_addrs* addrs;
    int err=0;
    uint8_t flag;

    err = f_get_boot_flag(opts, &flag);
    if (!err)
    {
        flag^=1;
        addrs =  &f_firm_addrs[flag];
        printf("Burn firmware for flag = %d\n", flag);
    }
    if (!err)
    {
        /* защищаем область 0x180000 или 0x1F0000 в зависимости от флага */
        err = opts->intf->flash_set_prot(flag ? SST25_STATUS_BP2 :
                                             SST25_STATUS_BP0);
    }

    if (err)
         f_print_err(err, "Cannot set flash protections bits!");

    /* стираем область памяти под прошивку */
    if (!err)
    {
        printf("Start flash erase (0x%x-0x%x)\n", addrs->prefix, addrs->last);
        err = f_erase(opts, addrs->prefix, addrs->last);
    }

    if (!err)
    {
         printf("Start burn firmware\n");
         err = f_write(opts, &pref, addrs->prefix, size);
    }

    if (!err)
    {
        printf("Start firmware verifying\n");
        err = f_firm_check(opts, &pref, addrs->prefix, size);
    }

    if (!err && !opts->reserv_only)
    {
        /* устанавливаем загрузочный флаг в 1, чтобы указать, что
           грузимся из нужной области */
        printf("Start setting bootflag\n");
        err = f_set_boot_flag(opts, flag);
    }

    return err;
}






static int f_write_firm(t_l502_burn_opt* opts)
{
    int err = 0;
    int stop_err;
    t_firm_prefix pref;


    if (opts->intf->check_dev)
    {
        err = opts->intf->check_dev(opts);
        if (err)
            f_print_err(err, "Invalid device!");
    }

    if (!err && opts->intf->check_ver && !opts->ver_dont_check)
    {
        char* ver = firm_get_version(opts->firm_info);
        err = opts->intf->check_ver(opts, ver);
        if (err)
            f_print_err(err, "Device has firmware with more recent version");
    }

    if (!err)
    {

        memset(&pref, 0, sizeof(pref));

        firm_get_size(opts->firm_info, &pref.size);



        if (!err)
        {
            err = f_write_firm_par(opts, pref);
            if (!err && !opts->main_only && !opts->reserv_only)
                err = f_write_firm_par(opts, pref);
            if (!err && !opts->main_only && !opts->reserv_only)
                err = f_set_boot_flag(opts, 0);
        }

        /* защищаем всю область, включая переписанную прошивку */
        stop_err = opts->intf->flash_set_prot(SST25_STATUS_BP0 | SST25_STATUS_BP1 |
                                              SST25_STATUS_BP2);
        if (stop_err)
        {
            f_print_err(stop_err, "Cannot set flash protections bits!");
            if (!err)
                err = stop_err;
        }
    }

    return err;
}


static int f_proc_op(t_l502_burn_opt* opts)
{
    int32_t err = opts->intf->open(opts);
    if (err)
        f_print_err(err, "Cannot open device");
    else
    {
        switch (opts->op)
        {
            case L502BURN_OP_WRITE:
                err = f_write_firm(opts);
                break;
            default:
                err = L502BURN_ERR_INVALID_USAGE;
                fprintf(stderr, "Invalid command!\n");
                break;
            //case L502BURN_OP_READ:
              //  err = f_read_flash(&par, &hnd);
               // break;
        }



        opts->intf->close();
    }
    return err;
}

int main(int argc, char **argv)
{
    t_l502_burn_opt opts;
    int err, out;
    err = f_parse_options(&opts, argc, argv, &out);
    if (!err && !out)
    {

        err = firm_check_sign(opts.filename);
        if (err)
            fprintf(stderr, "Signature check faild!\n");

        if (!err)
        {
            err = firm_parse_init(opts.filename, &opts.firm_info);
            if (err)
                fprintf(stderr, "Invalid fimware format!\n");

            if (!err)
            {
                if (opts.all)
                {
                    opts.intf->exec_for_all(&opts, f_proc_op);
                }
                else
                {
                    f_proc_op(&opts);
                }

                firm_parse_clean(opts.firm_info);
            }
        }
    }
    return err;
}




